<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kompare_part" version="7" translationDomain="kompare">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="file_save"/>
    <Action name="file_save_all"/>
    <Action name="file_save_as"/>
    <Action name="file_save_diff"/>
    <Separator/>
    <Action name="file_refreshdiff"/>
    <Action name="file_swap"/>
    <Action name="file_diffstats"/>
    <Separator/>
    <Action name="file_print"/>
    <Action name="file_print_preview"/>
  </Menu>
  <Menu name="difference"><text>&amp;Difference</text>
    <Action name="difference_unapplyall"/>
    <Action name="difference_unapply"/>
    <Action name="difference_apply"/>
    <Action name="difference_applyall"/>
    <Separator/>
    <Action name="difference_previousfile"/>
    <Action name="difference_nextfile"/>
    <Separator/>
    <Action name="difference_previous"/>
    <Action name="difference_next"/>
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
    <Action name="options_configure"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar">
  <Action name="file_save"/>
  <Action name="file_save_all"/>
  <Separator/>
  <Action name="difference_previousfile"/>
  <Action name="difference_nextfile"/>
  <Separator/>
  <Action name="difference_previous"/>
  <Action name="difference_next"/>
  <Separator/>
  <Action name="difference_unapplyall"/>
  <Action name="difference_unapply"/>
  <Action name="difference_apply"/>
  <Action name="difference_applyall"/>
</ToolBar>
<Menu name="mainPopUp">
  <Action name="difference_unapplyall"/>
  <Action name="difference_unapply"/>
  <Action name="difference_apply"/>
  <Action name="difference_applyall"/>
  <Separator/>
  <Action name="difference_previousfile"/>
  <Action name="difference_nextfile"/>
  <Separator/>
  <Action name="difference_previous"/>
  <Action name="difference_next"/>
</Menu>
</gui>
